@@ -26,24 +26,15 @@
 #define LAYOUT_NAME "/.gpe/contacts-layout.xml"
 #define DEFAULT_STRUCTURE PREFIX "/share/gpe-contacts/default-layout.xml"
 
-#undef USE_USQLD
-
-#ifdef USE_USQLD
-#include <usql.h>
-static usqld_conn *db;
-#else
 #include <sqlite.h>
 static sqlite *db;
-#endif
 
-extern GtkWidget *clist;
+extern void migrate_old_categories (sqlite *db);
 
 static const char *schema_str = "create table contacts (urn INTEGER NOT NULL, tag TEXT NOT NULL, value TEXT NOT NULL);";
 
 static const char *schema2_str = "create table contacts_urn (urn INTEGER PRIMARY KEY);";
 
-static const char *schema3_str = "create table contacts_category (id INTEGER PRIMARY KEY, description TEXT);";
-
 // this one is for config data
 static const char *schema4_str = "create table contacts_config (id INTEGER PRIMARY KEY,	cgroup INTEGER NOT NULL, cidentifier TEXT NOT NULL, cvalue TEXT);";
 
@@ -53,51 +44,6 @@ db_open (void)
   /* open persistent connection */
   char *errmsg;
   
-#ifdef USE_USQLD
-  int ret;
-
-  db = usqld_connect ("localhost", "contacts", &errmsg);
-  
-  if (db == NULL) 
-    {
-      gpe_error_box (errmsg);
-      free (errmsg);
-      return -1;
-    }
-  
-  ret = usqld_exec (db, schema_str, NULL, NULL, &errmsg);
-  if (ret != SQLITE_OK)
-    {
-      printf ("err: %s\n", errmsg);
-      free (errmsg);
-    }
-  ret = usqld_exec (db, schema2_str, NULL, NULL, &errmsg);
-  if (ret != SQLITE_OK)
-    {
-      printf ("err: %s\n", errmsg);
-      free (errmsg);
-    }
-  ret = usqld_exec (db, schema3_str, NULL, NULL, &errmsg);
-  if (ret != SQLITE_OK)
-    {
-      printf ("err: %s\n", errmsg);
-      free (errmsg);
-    }
-  // if we can create this table, we should add some defaults
-  ret = usqld_exec (db, schema4_str, NULL, NULL, &errmsg);
-  if (ret != SQLITE_OK)
-    {
-      printf ("err: %s\n", errmsg);
-      free (errmsg);
-  }
-  else
-    {
-      db_add_config_values (CONFIG_PANEL, _("Name"), "NAME");
-      db_add_config_values (CONFIG_PANEL, _("Phone"), "HOME.TELEPHONE");
-      db_add_config_values (CONFIG_PANEL, _("EMail"), "HOME.EMAIL");
-    }
-
-#else
   char *buf;
   size_t len;
   char *home = getenv ("HOME");
@@ -121,7 +67,6 @@ db_open (void)
 
   sqlite_exec (db, schema_str, NULL, NULL, NULL);
   sqlite_exec (db, schema2_str, NULL, NULL, NULL);
-  sqlite_exec (db, schema3_str, NULL, NULL, NULL);
   // if we can create this table, we should add some defaults
   if (sqlite_exec (db, schema4_str, NULL, NULL, NULL) == SQLITE_OK)
     {
@@ -129,7 +74,11 @@ db_open (void)
       db_add_config_values (CONFIG_PANEL, _("Phone"), "HOME.TELEPHONE");
       db_add_config_values (CONFIG_PANEL, _("EMail"), "HOME.EMAIL");
     }
-#endif
+
+  gpe_pim_categories_init ();
+
+  migrate_old_categories (db);
+
   return 0;
 }
 
@@ -207,13 +156,8 @@ gboolean
 new_person_id (guint * id)
 {
   char *err;
-#ifdef USE_USQLD	
-  int r = usqld_exec (db, "insert into contacts_urn values (NULL)",
-		       NULL, NULL, &err);
-#else	
   int r = sqlite_exec (db, "insert into contacts_urn values (NULL)",
 		       NULL, NULL, &err);
-#endif
   if (r)
     {
       gpe_error_box (err);
@@ -221,11 +165,7 @@ new_person_id (guint * id)
       return FALSE;
     }
 
-#ifdef USE_USQLD	
-  *id = usqld_last_insert_rowid (db);
-#else
   *id = sqlite_last_insert_rowid (db);
-#endif	
   return TRUE;
 }
 
@@ -239,9 +179,7 @@ static int
 read_name (void *arg, int argc, char **argv, char **names)
 {
   struct person *p = (struct person *)arg;
-
-  //if (strcasecmp (argv[0], "NAME") == 0)
-    p->name = g_strdup (argv[1]);
+  p->name = g_strdup (argv[1]);
 
   return 0;
 }
@@ -253,16 +191,9 @@ read_one_entry (void *arg, int argc, char **argv, char **names)
   GSList **list = (GSList **) arg;
 
   p->id = atoi (argv[0]);
-/*
-#ifdef USE_USQLD	
-  usqld_exec_printf (db, "select tag,value from contacts where (urn=%d) and ((tag='NAME') or (tag='name'))",
-		     read_name, p, NULL, p->id);
-#else
   sqlite_exec_printf (db, "select tag,value from contacts where (urn=%d) and ((tag='NAME') or (tag='name'))",
 		      read_name, p, NULL, p->id);
-#endif  
-*/
-  p->name = g_strdup (argv[1]);
+
   if (p->name)
     *list = g_slist_prepend (*list, p);
   else
@@ -278,13 +209,8 @@ db_get_entries (void)
   char *err;
   int r;
 
-#ifdef USE_USQLD	
-  r = usqld_exec (db, "select urn from contacts_urn",
-		  read_one_entry, &list, &err);
-#else
   r = sqlite_exec (db, "select urn from contacts_urn",
 		   read_one_entry, &list, &err);
-#endif
 
   if (r)
     {
@@ -312,11 +238,7 @@ db_get_by_uid (guint uid)
   int r;
 
   p->id = uid;
-#ifdef USE_USQLD	
-  r = usqld_exec_printf (db, "select tag,value from contacts where urn=%d",
-#else
   r = sqlite_exec_printf (db, "select tag,value from contacts where urn=%d",
-#endif  
   read_entry_data, p, &err, uid);
   if (r)
     {
@@ -336,41 +258,23 @@ db_delete_by_uid (guint uid)
   gchar *err;
   gboolean rollback = FALSE;
 
-#ifdef USE_USQLD	
-  r = usqld_exec (db, "begin transaction", NULL, NULL, &err);
-#else
   r = sqlite_exec (db, "begin transaction", NULL, NULL, &err);
-#endif
   if (r)
     goto error;
 
   rollback = TRUE;
 
-#ifdef USE_USQLD	
-  r = usqld_exec_printf (db, "delete from contacts where urn='%d'",
-			 NULL, NULL, &err, uid);
-#else
   r = sqlite_exec_printf (db, "delete from contacts where urn='%d'",
 			  NULL, NULL, &err, uid);
-#endif
   if (r)
     goto error;
   
-#ifdef USE_USQLD	
-  r = usqld_exec_printf (db, "delete from contacts_urn where urn='%d'",
-			 NULL, NULL, &err, uid);
-#else
   r = sqlite_exec_printf (db, "delete from contacts_urn where urn='%d'",
 			  NULL, NULL, &err, uid);
-#endif
   if (r)
     goto error;
 
-#ifdef USE_USQLD	
-  r = usqld_exec (db, "commit transaction", NULL, NULL, &err);
-#else
   r = sqlite_exec (db, "commit transaction", NULL, NULL, &err);
-#endif
   if (r)
     goto error;
 
@@ -378,11 +282,7 @@ db_delete_by_uid (guint uid)
 
 error:
   if (rollback)
-#ifdef USE_USQLD	
-    usqld_exec (db, "rollback transaction", NULL, NULL, NULL);
-#else
     sqlite_exec (db, "rollback transaction", NULL, NULL, NULL);
-#endif  
   gpe_error_box (err);
   free (err);
   return FALSE;
@@ -459,11 +359,7 @@ commit_person (struct person *p)
 
   time (&modified);
 
-#ifdef USE_USQLD	
-  r = usqld_exec (db, "begin transaction", NULL, NULL, &err);
-#else
   r = sqlite_exec (db, "begin transaction", NULL, NULL, &err);
-#endif	
   if (r)
     goto error;
 
@@ -471,13 +367,8 @@ commit_person (struct person *p)
 
   if (p->id)
     {
-#ifdef USE_USQLD		
-      r = usqld_exec_printf (db, "delete from contacts where urn='%d'",
-			     NULL, NULL, &err, p->id);
-#else
       r = sqlite_exec_printf (db, "delete from contacts where urn='%d'",
 			      NULL, NULL, &err, p->id);
-#endif		
       if (r)
 	goto error;
     }
@@ -485,11 +376,7 @@ commit_person (struct person *p)
     {
       if (new_person_id (&p->id) == FALSE)
 	{
-#ifdef USE_USQLD		
-	  usqld_exec (db, "rollback transaction", NULL, NULL, NULL);
-#else
 	  sqlite_exec (db, "rollback transaction", NULL, NULL, NULL);
-#endif		
 	  return FALSE;
 	}
     }
@@ -499,15 +386,9 @@ commit_person (struct person *p)
       struct tag_value *v = iter->data;
       if (v->value && v->value[0])
 	{
-#ifdef USE_USQLD		
-	  r = usqld_exec_printf (db,
-				  "insert into contacts values(%d,'%q','%q')",
-				 NULL, NULL, &err, p->id, v->tag, v->value);
-#else
 	  r = sqlite_exec_printf (db,
 				  "insert into contacts values(%d,'%q','%q')",
 				  NULL, NULL, &err, p->id, v->tag, v->value);
-#endif		
 	}
       if (r)
 	goto error;
@@ -518,11 +399,7 @@ commit_person (struct person *p)
 			  NULL, NULL, &err, p->id, modified))
     goto error;
 
-#ifdef USE_USQLD		
-  r = usqld_exec (db, "commit transaction", NULL, NULL, &err);
-#else
   r = sqlite_exec (db, "commit transaction", NULL, NULL, &err);
-#endif		
   if (r)
     goto error;
 
@@ -530,11 +407,7 @@ commit_person (struct person *p)
 
 error:
   if (rollback)
-#ifdef USE_USQLD		
-    usqld_exec (db, "rollback transaction", NULL, NULL, NULL);
-#else
     sqlite_exec (db, "rollback transaction", NULL, NULL, NULL);
-#endif		
   gpe_error_box (err);
   free (err);
   return FALSE;
@@ -544,17 +417,10 @@ gboolean
 db_insert_category (gchar * name, guint * id)
 {
   char *err;
-#ifdef USE_USQLD		
-  int r =
-    usqld_exec_printf (db,
-		       "insert into contacts_category values (NULL, '%q')",
-			      NULL, NULL, &err, name);
-#else
   int r =
     sqlite_exec_printf (db,
 			"insert into contacts_category values (NULL, '%q')",
 			      NULL, NULL, &err, name);
-#endif		
   if (r)
     {
       gpe_error_box (err);
@@ -562,11 +428,7 @@ db_insert_category (gchar * name, guint * id)
       return FALSE;
     }
   
-#ifdef USE_USQLD 
-  *id = usqld_last_insert_rowid (db);
-#else
   *id = sqlite_last_insert_rowid (db);
-#endif
   return TRUE;
 }
  
@@ -574,13 +436,8 @@ gboolean
 db_delete_category (guint id)
 {
   char *err;
-#ifdef USE_USQLD		
-  int r = usqld_exec_printf (db, "delete from contacts_category where id=%d", 
-			      NULL, NULL, &err, id);
-#else
   int r = sqlite_exec_printf (db, "delete from contacts_category where id=%d", 
 			      NULL, NULL, &err, id);
-#endif		
   if (r)
     {
       gpe_error_box (err);
@@ -592,44 +449,22 @@ db_delete_category (guint id)
 }
 
 static int
-load_one_attribute (void *arg, int argc, char **argv, char **names)
+read_one_entry_alpha (void *arg, int argc, char **argv, char **names)
 {
-  if (argc == 2)
-    {
-      GSList **list = (GSList **) arg;
-      struct category *c = g_malloc (sizeof (struct category));
+  struct person *p = new_person ();
+  GSList **list = (GSList **) arg;
 
-      c->id = atoi (argv[0]);
-      c->name = g_strdup (argv[1]);
+  p->id = atoi (argv[0]);
+  p->name = g_strdup (argv[1]);
 
-      *list = g_slist_prepend (*list, c);
-    }
+  if (p->name)
+    *list = g_slist_prepend (*list, p);
+  else
+    discard_person (p);
 
   return 0;
 }
 
-GSList *
-db_get_categories (void)
-{
-  GSList *list = NULL;
-  char *err;
-  if (
-#ifdef USE_USQLD		
-      usqld_exec
-#else
-      sqlite_exec
-#endif		
-      (db, "select id,description from contacts_category",
-       load_one_attribute, &list, &err))
-    {
-      fprintf (stderr, "sqlite: %s\n", err);
-      free (err);
-      return NULL;
-    }
-  
-  return list;
-}
-
 GSList *
 db_get_entries_alpha (const gchar * alphalist)
 {
@@ -691,13 +526,8 @@ db_get_entries_alpha (const gchar * alphalist)
 			     "where ((( tag = 'NAME') or ( tag = 'name')) and%s(%s)) "
 			     "order by value desc", modifier, tmp);
   g_free (tmp);
-#ifdef USE_USQLD	
-  r = usqld_exec (db, command,
-		  read_one_entry, &list, &err);
-#else
   r = sqlite_exec (db, command,
-		   read_one_entry, &list, &err);
-#endif
+		   read_one_entry_alpha, &list, &err);
 
   if (r)
     {
@@ -715,13 +545,8 @@ db_get_tag_list (gchar *** list)
   gchar *err;
   gint r, c;
 
-#ifdef USE_USQLD
-  usqld_get_table (db, "select distinct tag from contacts", list, &r, &c,
-		   &err);
-#else
   sqlite_get_table (db, "select distinct tag from contacts", list, &r, &c,
 		    &err);
-#endif
   if (err)
     g_free (err);
   return r;
@@ -734,20 +559,16 @@ db_get_config_values (gint group, gchar *** list)
   gchar *err, *statement;
   gint r, c;
 
-  statement = g_malloc (sizeof (gchar) * 100);
-  sprintf (statement,
-	   "select cidentifier,cvalue from contacts_config where cgroup=%i",
-	   group);
-#ifdef USE_USQLD
-  usqld_get_table (db, statement, list, &r, &c, &err);
-#else
+  statement = g_strdup_printf ("select cidentifier,cvalue from contacts_config where cgroup=%i",
+			       group);
+
   sqlite_get_table (db, statement, list, &r, &c, &err);
-#endif
   if (err)
     {
       fprintf (stderr,"e: %s\n", err);
       g_free (err);
     }
+
   g_free (statement);
   return r;
 }
@@ -758,15 +579,9 @@ db_add_config_values (gint group, gchar * identifier, gchar * value)
   gchar *err;
   gint r, c;
   gchar **list;
-#ifdef USE_USQLD
-  usqld_get_table_printf (db,
-			  "insert into contacts_config (cgroup,cidentifier,cvalue) values(%i,'%q','%q')",
-			  &list, &r, &c, &err, group, identifier, value);
-#else
   sqlite_get_table_printf (db,
 			   "insert into contacts_config (cgroup,cidentifier,cvalue) values(%i,'%q','%q')",
 			   &list, &r, &c, &err, group, identifier, value);
-#endif
   if (err)
     {
       fprintf (stderr,"e: %s\n", err);
@@ -774,11 +589,7 @@ db_add_config_values (gint group, gchar * identifier, gchar * value)
     }
   if (r)
     {
-#ifdef USE_USQLD
-      usqld_free_table (list);
-#else
       sqlite_free_table (list);
-#endif
     }
 }
 
@@ -790,15 +601,9 @@ db_delete_config_values (gint group, gchar * identifier)
   gint r = 0;
   gint c = 0;
   gchar **list;
-#ifdef USE_USQLD
-  usqld_get_table_printf (db,
-			  "delete from contacts_config where (cgroup=%i) and (cidentifier='%q')",
-			  &list, &r, &c, &err, group, identifier);
-#else
   sqlite_get_table_printf (db,
 			   "delete from contacts_config where (cgroup=%i) and (cidentifier='%q')",
 			   &list, &r, &c, &err, group, identifier);
-#endif
   if (err)
     {
       fprintf (stderr,"e: %s\n", err);
@@ -806,11 +611,7 @@ db_delete_config_values (gint group, gchar * identifier)
     }
   if (r)
     {
-#ifdef USE_USQLD
-      usqld_free_table (list);
-#else
       sqlite_free_table (list);
-#endif
     }
 }
 
@@ -822,15 +623,9 @@ db_get_config_tag (gint group, const gchar * tagname)
   gchar **list = NULL;
   gint r, c;
 
-#ifdef USE_USQLD
-  usqld_get_table_printf (db,
-			  "select cvalue from contacts_config where (cgroup=%i) and (cidentifier='%q')",
-			  &list, &r, &c, &err, group, tagname);
-#else
   sqlite_get_table_printf (db,
 			   "select cvalue from contacts_config where (cgroup=%i) and (cidentifier='%q')",
 			   &list, &r, &c, &err, group, tagname);
-#endif
   if (err)
     {
       fprintf (stderr,"e: %s\n", err);
@@ -840,11 +635,7 @@ db_get_config_tag (gint group, const gchar * tagname)
   if (list != NULL)
     {
       err = g_strdup (list[1]);
-#ifdef USE_USQLD
-      usqld_free_table (list);
-#else
       sqlite_free_table (list);
-#endif
     }
   return err;
 }
@@ -852,9 +643,6 @@ db_get_config_tag (gint group, const gchar * tagname)
 void
 db_free_result (char **table)
 {
-#ifdef USE_USQLD
-  usqld_free_table (table);
-#else
   sqlite_free_table (table);
-#endif
 }
+
